import { UndoManager } from 'ace-builds';
import { flatten, map, not, compose, filter } from 'ramda';
import { aceGrayTheme } from './aceGrayTheme';
var aceTheme = 'ace/theme/theme-gray';
export var initAceEditor = function (editor) {
    window.ace.define(aceTheme, ['acequire', 'exports', 'ace/lib/dom'], function (acequire, exports) {
        exports.cssClass = 'ace-gray';
        exports.cssText = aceGrayTheme;
        var dom = acequire('ace/lib/dom');
        dom.importCssString(exports.cssText, exports.cssClass);
    });
    editor.setTheme(aceTheme);
    var session = editor.getSession();
    session.setUndoManager(new UndoManager());
    editor.container.style.lineHeight = '1.5';
    editor.renderer.setPadding(16);
    editor.renderer.setScrollMargin(16, 0, 0, 0);
    editor.navigateLineEnd();
    editor.setOptions({
        showFoldWidgets: false,
        enableBasicAutocompletion: true,
        enableLiveAutocompletion: false,
        showPrintMargin: false,
        tabSize: 2,
        fontSize: 14,
        useSoftTabs: true,
    });
};
export var getRules = function (variables) {
    var rules = {
        start: [],
    };
    Object.keys(variables).forEach(function (key, index) {
        if (variables[key].length > 0) {
            rules.start.push({
                token: "variable.other.kilang." + key,
                regex: createVariableRule(variables[key]),
            });
            rules.start.push({
                token: "variable.other.constant.kilang." + key,
                regex: createConstantRule(variables[key]),
            });
        }
    });
    return rules;
};
export var insert = function (src, text, at) {
    if (at === void 0) { at = 0; }
    return src.slice(0, at) + text + src.slice(at);
};
var toRegexString = function (strings) {
    return strings.map(function (string) { return string + "\\b"; }).join('|') || null;
};
var toTextVariable = function (raw) { return (raw[0] === '/' ? raw.slice(1) : raw); };
var isElixirConstant = function (str) {
    return str[0] === str && str[0].toUpperCase();
};
var callRFlatten = function (value) { return flatten(value); };
var isElixirVariable = compose(not, isElixirConstant);
var prepareValues = compose(map(toTextVariable), callRFlatten);
var createVariableRule = compose(toRegexString, filter(isElixirVariable), prepareValues);
var createConstantRule = compose(toRegexString, filter(isElixirConstant), prepareValues);
